<?php

namespace FiloBlu\AttributeRelated\Block;

use Magento\Catalog\Block\Product\ImageBuilder;
use Magento\Catalog\Helper\Image;
use Magento\Catalog\Model\Product\Attribute\Repository;
use Magento\Catalog\Model\ResourceModel\Product\CollectionFactory;
use Magento\CatalogInventory\Helper\Stock;
use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Framework\Exception\NoSuchEntityException;
use Magento\Framework\Registry;
use Magento\Framework\View\Element\Template;
use Magento\Store\Model\ScopeInterface;
use Magento\Store\Model\StoreManagerInterface;
use Magento\Swatches\Helper\Media;
use Magento\Swatches\Model\ResourceModel\Swatch\Collection;
use Magento\Swatches\Model\Swatch;

/**
 * Class AttributeRelated
 * @package FiloBlu\AttributeRelated\Block
 */
class AttributeRelated extends Template
{
    /**
     * @var string
     */
    const ALLOWED_PRODUCT_TYPE_PATH = 'filoblu_attributerelated/general/allowed_product_types';
    /**
     * @var string
     */
    const PRODUCT_LIMIT_PATH = 'filoblu_attributerelated/general/product_limit';
    /**
     * @var string
     */
    const ENABLED_ATTRIBUTE_RELATED_PATH = 'filoblu_attributerelated/general/enabled_attribute_related';
    /**
     * @var string
     */
    const ATTRIBUTE_LABEL_PATH = 'filoblu_attributerelated/general/swatch_attribute_label';
    /**
     * @var string
     */
    const SWATCH_ATTRIBUTE_PATH = 'filoblu_attributerelated/general/swatch_attribute';
    /**
     * @var string
     */
    const ATTRIBUTE_RELATION_PATH = 'filoblu_attributerelated/general/attribute_relation';
    /**
     * @var string
     */
    const HIDE_OUT_OF_STOCK_PATH = 'filoblu_attributerelated/general/hide_out_of_stock';
    /**
     * @var
     */
    public $currentProduct;

    /**
     * @var StoreManagerInterface
     */
    protected $imageBuilder;
    /**
     * @var Registry
     */
    protected $registry;
    /**
     * @var CollectionFactory
     */
    protected $productCollectionFactory;
    /**
     * @var Stock
     */
    protected $stockFilter;
    /**
     * @var Media
     */
    protected $swatchHelperMedia;
    /**
     * @var array
     */
    protected $lookupWatch = [];
    /**
     * @var Image|mixed
     */
    protected $imageHelper;
    /**
     * @var string
     */
    protected $defaultSwatchThumb;
    /**
     * @var string
     */
    protected $defaultSwatchImage;
    /**
     * @var Collection
     */
    private $swatchCollection;

    /**
     * @var Repository
     */
    private $attributeRepository;

    /**
     * AttributeRelated constructor.
     * @param ScopeConfigInterface $scopeConfig
     * @param ImageBuilder $_imageBuilder
     * @param Registry $registry
     * @param CollectionFactory $productCollectionFactory
     * @param Stock $stockFilter
     * @param Media $swatchHelperMedia
     * @param Repository $attributeRepository
     * @param Collection $swatchCollection
     * @param Template\Context $context
     * @param Image $imageHelper
     * @param array $data
     */
    public function __construct(
        ScopeConfigInterface $scopeConfig,
        ImageBuilder $_imageBuilder,
        Registry $registry,
        CollectionFactory $productCollectionFactory,
        Stock $stockFilter,
        Media $swatchHelperMedia,
        Repository $attributeRepository,
        Collection $swatchCollection,
        Template\Context $context,
        Image $imageHelper,
        array $data = []
    )
    {
        $this->_scopeConfig = $scopeConfig;
        $this->imageBuilder = $_imageBuilder;
        $this->registry = $registry;
        $this->productCollectionFactory = $productCollectionFactory;
        $this->stockFilter = $stockFilter;
        $this->swatchHelperMedia = $swatchHelperMedia;
        $this->attributeRepository = $attributeRepository;
        $this->swatchCollection = $swatchCollection;
        $this->imageHelper = $imageHelper;
        parent::__construct($context, $data);
    }

    /**
     * @return mixed
     */
    public function isAttributeRelatedEnabled()
    {
        return $this->_scopeConfig->getValue(
            self::ENABLED_ATTRIBUTE_RELATED_PATH,
            ScopeInterface::SCOPE_STORE);
    }

    /**
     * @return mixed
     */
    public function getAttributeLabel()
    {
        return $this->_scopeConfig->getValue(
            self::ATTRIBUTE_LABEL_PATH,
            ScopeInterface::SCOPE_STORE);
    }

    /**
     * @return \Magento\Catalog\Model\ResourceModel\Product\Collection
     * @throws NoSuchEntityException
     */
    public function getAttributeRelatedCollection()
    {
        $collection = $this->productCollectionFactory->create()
            ->addAttributeToSelect('*')
            ->addAttributeToFilter($this->getRelationshipAttribute(), $this->getCurrentProductRelationAttributeValue())
            ->addAttributeToSelect('name')
            ->addAttributeToFilter('type_id', ['in' => $this->getAllowedProductTypes()])
            ->setPageSize($this->getProductLimit())
            ->addStoreFilter();

        if ($this->hideOutOfStock()) {
            $this->stockFilter->addInStockFilterToCollection($collection);
        }

        $swatchAttribute = $this->getSwatchAttribute();
        foreach ($collection->getItems() as $item) {
            $item->setData('fb_attribute_related_swatch', $this->lookUpAttribute($item->getData($swatchAttribute)));
        }

        return $collection;
    }

    /**
     * @return mixed
     */

    public function getRelationshipAttribute()
    {
        return $this->_scopeConfig->getValue(
            self::ATTRIBUTE_RELATION_PATH,
            ScopeInterface::SCOPE_STORE);
    }

    /**
     * @return mixed
     */
    public function getCurrentProductRelationAttributeValue()
    {
        return $this->getCurrentProduct()->getData($this->getRelationshipAttribute());
    }

    /**
     * Set Current Product From Registry
     */
    protected function getCurrentProduct()
    {
        if ($this->currentProduct) {
            return $this->currentProduct;
        }

        return $this->currentProduct = $this->registry->registry('current_product');
    }

    /**
     * @return mixed
     */
    public function getAllowedProductTypes()
    {
        return $this->_scopeConfig->getValue(
            self::ALLOWED_PRODUCT_TYPE_PATH,
            ScopeInterface::SCOPE_STORE);
    }

    /**
     * @return int
     */
    public function getProductLimit()
    {
        return (int)$this->_scopeConfig->getValue(
            self::PRODUCT_LIMIT_PATH,
            ScopeInterface::SCOPE_STORE);
    }

    /**
     * @return  bool
     */
    public function hideOutOfStock()
    {
        return $this->_scopeConfig->getValue(
            self::HIDE_OUT_OF_STOCK_PATH,
            ScopeInterface::SCOPE_STORE);

    }

    /**
     * @return mixed
     */
    public function getSwatchAttribute()
    {
        return $this->_scopeConfig->getValue(
            self::SWATCH_ATTRIBUTE_PATH,
            ScopeInterface::SCOPE_STORE);
    }

    /**
     * @param $attributeValue
     * @return array|mixed
     * @throws NoSuchEntityException
     */
    public function lookUpAttribute($attributeValue)
    {
        if ($attributeValue === null) {
            return [];
        }

        if (!$this->defaultSwatchImage) {
            $this->defaultSwatchImage = $this->imageHelper->getDefaultPlaceholderUrl('swatch_image');
        }

        if (!$this->defaultSwatchThumb) {
            $this->defaultSwatchThumb = $this->imageHelper->getDefaultPlaceholderUrl('swatch_thumb');
        }

        if (empty($this->lookupWatch)) {

            foreach ($this->swatchCollection->getItems() as $swatch) {

                $value = $swatch->getValue();

                switch ($swatch->getType()) {

                    case Swatch::SWATCH_TYPE_TEXTUAL:
                        $value = $value ?: 'Not set';
                        $type = 'text';
                        break;
                    case Swatch::SWATCH_TYPE_VISUAL_IMAGE:
                        $value = $value ?: $this->defaultSwatchImage;
                        $type = 'image';
                        break;
                    case Swatch::SWATCH_TYPE_VISUAL_COLOR:
                        $value = $value ?: '#050505';
                        $type = 'color';
                        break;
                    case  Swatch::SWATCH_TYPE_EMPTY;
                    default:
                        $value = $this->defaultSwatchImage;
                        $type = 'image';
                }

                $this->lookupWatch[$swatch->getOptionId()] = ['type' => $type, 'value' => $value];
            }
        }
        $this->attributeRepository->get($this->getSwatchAttribute());
        return $this->lookupWatch[$attributeValue];
    }
}
